
package org.sablecc.sablecc;

import java.util.*;

/** This class contains all the state that is required to genereate
  * a parser. All the state is already "calculated and resolved" so
  * no grammar analysis is neccessary. */
public class ParserData {
  public String package_name;

  /** Class representing what is neede for a token. */
  public static class Token {
    Token (String name, String ename, String value, int index)
    {
      this.name = name;
      this.ename = ename;
      this.value = value;
      this.index = index;
      if ( verbose )
        System.out.println ("new Token (" + name + ", " + ename + ", " + value + ", " + index + ")");
    }

    /** Name of token, like 'token'. */
    public String name;
    /** Encoded name, like 'TToken'. */
    public String ename;
    /** Value of a fixed token, null if variable token. */
    public String value;
    /** Index of the token for parser. */
    public int index;

    public Map transitions = new HashMap();
  }

  public final List token_list = new LinkedList ();
  public int eof_index;

  /** Vector[Vector[Integer]] */
  public Vector lexer_accept;
  /** Vector[Vector[Vector[int[]]]]] */
  public Vector lexer_goto_table;
//  public int[][][][] lexer_goto_table;

  /** Class representing what is neede for a lexical analyzer state. */
  public static class LexerState {
    LexerState (String name, int id)
    {
      this.name = name;
      this.id = id;
      if ( verbose )
        System.out.println ("new LexerState (" + name + ", " + id + ")");
    }

    public String name;     /* State name, like INITIAL */
    public int id;          /* State id */
  };

  public final List state_list = new LinkedList ();

  /** Class representing what is neede for a alternative element. */
  public static class Elem {
    Elem (String name, String ename, boolean is_token, String type, String etype, String modifier)
    {
      this.name = name;
      this.ename = ename;
      this.is_token = is_token;
      this.type = type;
      this.etype = etype;
      this.modifier = modifier;
      if ( "*".equals(modifier) || "+".equals(modifier) )
          this.is_list = true;
      else
          this.is_list = false;
      if ( verbose )
        System.out.println ("new Elem (" + name + ", " + ename + ", " + is_token + ", " +
          type + ", " + etype + ", " + modifier + ")");
    }

    public String name;
    public String ename;
    public boolean is_token;
    public String type;
    public String etype;
    public String modifier; /* Modifier, "*", "+", "?" or null */
    public boolean is_list;
  }

  /** Class representing what is needed for an alternative. */
  public static class Alt {
    Alt (String n, String en)
    {
      name = n;
      ename = en;

      if (verbose)
        System.out.println ("new Alt (" + name + ", " + ename + ")");
    }

    public String name;   /* Name of the alternative. */
    public String ename;
    public final List elems = new LinkedList ();
  }

  /** Class representing what is needed for a production. */
  public static class Production {
    Production (String n, String en)
    {
      name = n;
      ename = en;

      if (verbose)
        System.out.println ("new Production (" + name + ", " + ename + ")");
    }

    public String name;
    public String ename;
    public final List alts = new LinkedList ();
  }

  public final List production_list = new LinkedList ();

  /** Vector[Vector[int[3]]] */
  public Vector parser_action_table;
  /** Vector[Vector[int[2]]] */
  public Vector parser_goto_table;
  /** Vector[String] */
  public Vector parser_error_messages;
  /** Vector[Integer] */
  public Vector parser_errors;

  /** This is a really silly "language", i couldn't figure a better way.
    * We probably need goto and conditionals to be turing complete. */
  public static class Action {
    Action (int cmd, String result, int index, String[] arg, String etype)
    {
      if ( arg == null ) arg = new String[] {};
      this.cmd = cmd;
      this.result = result;
      this.index = index;
      this.arg = arg;
      this.etype = etype;

      if (verbose)
      {
        String a = "";
        if ( arg != null )
        for ( int i = 0; i < arg.length; i++ ) { a += arg[i]; if ( i + 1 < arg.length ) a += ", "; }
        System.out.println ("new Action (" + cmd2str[cmd] + ", " + result + ", (" + a + "), " + index + ", " + etype + ")");
      }
    }

    public int cmd;                 // POP, FETCH, etc.
    public String result;           // result = cmd(arg1, ..)
    public int index;               // used for indexing at FETCH and FETCHLIST 
    public String arg[];
    public String etype;

    public final static int POP = 0;       // pop()-s a vector from stack (reduce) => result
    public final static int FETCHNODE = 1; // fetch node from vector arg[0] at pos index of type etype => result
    public final static int FETCHLIST = 2; // fetch list from vector arg[0] at pos index of type etype (list elems) => result
    public final static int ADDNODE = 3;   // add node arg[1] into list arg[0]
    public final static int ADDLIST = 4;   // add all elements from list arg[1] into list arg[0] (both are lists)
    public final static int MAKELIST = 5;  // makes an empty list of type etype => result
    public final static int MAKENODE = 6;  // makes a new object with given encoded type in etype and arg[] => result
    public final static int RETURNNODE = 7;// adds arg[0] into return vector, can be null
    public final static int RETURNLIST = 8;// adds a list arg[0] into return vector

    private static String cmd2str[] = new String[] {"POP", "FETCHNODE", "FETCHLIST", "ADDNODE", "ADDLIST",
      "MAKELIST", "MAKENODE", "RETURNNODE", "RETURNLIST"};
  };

  public static class Rule {
    Rule (String ename, int index, int leftside)
    {
      this.ename = ename;
      this.index = index;
      this.leftside = leftside;

      if (verbose)
        System.out.println ("new Rule (" + ename + ", " + index + ", " + leftside + ")");
    }

    public String ename;
    public int index;                               // same as rule index/position in rules_list
    public int leftside;

    public final List actions = new LinkedList();   // a series of Action-s to be executed for this rule
  }

  /** Vector[Rule] */
  public final List rules_list = new LinkedList();

  static boolean verbose = false;
}

